@@ -3,8 +3,10 @@
 
 import click
 import json
+import logging
 import random
 import string
+import sys
 import time
 from pathlib import Path
 from flask import Flask, request, render_template, Response
@@ -247,7 +249,8 @@ class GetBuildStatus(Resource):
     @FakeRandom.generate_api_failures
     @check_token
     def get(self, build_key):
-        build = Build.get(build_key)
+        orig_build = Build.get(build_key)
+        build = orig_build.copy()
         if build["tuxbuild_status"] == "complete":
             return build
 
@@ -292,7 +295,7 @@ class GetBuildStatus(Resource):
             else:
                 build["errors_count"] = 1 + r.get_errors()
         Build.put(build_key, build, request.url_root)
-        return build
+        return orig_build
 
 
 api.add_resource(GetBuildStatus, "/v1/status/<build_key>")
@@ -376,6 +379,8 @@ def run(
     FakeRandom.infrastructure_failure_rate = infrastructure_failure_rate
     FakeRandom.api_failure_rate = api_failure_rate
     FakeRandom.duration = duration
+    if not sys.stdout.isatty():
+        logging.getLogger("werkzeug").setLevel(logging.WARN)
     app.run(port=port)
 
 
