@@ -4,7 +4,6 @@ import json
 import pytest
 import tuxbuild.build
 import requests
-from mock import patch
 import tuxbuild.exceptions
 
 
@@ -102,17 +101,27 @@ class TestGetRequest:
 
 
 @pytest.fixture
-def global_var():
-    pytest.start_time = 0
-
-
-def mock_get_status(self):
-    self.status["tuxbuild_status"] = "Queued"
+def start_time():
+    pytest.time = 0
 
 
 def mock_time():
-    pytest.start_time += 1
-    return pytest.start_time
+    return pytest.time
+
+
+def mock_sleep(n):
+    pytest.time += n
+    return pytest.time
+
+
+@pytest.fixture(autouse=True)
+def time(mocker, start_time):
+    return mocker.patch("time.time", side_effect=mock_time)
+
+
+@pytest.fixture(autouse=True)
+def sleep(mocker, start_time):
+    return mocker.patch("time.sleep", side_effect=mock_sleep)
 
 
 @pytest.fixture
@@ -128,20 +137,6 @@ def build_attrs():
     }
 
 
-@patch.object(tuxbuild.build.Build, "get_status", mock_get_status)
-@patch("time.time", mock_time)
-@patch("time.sleep")
-def test_get_status(stub_sleep, global_var, build_attrs):
-    b = tuxbuild.build.Build(**build_attrs)
-    b.status["tuxbuild_status"] = "Queued"
-    with pytest.raises(tuxbuild.exceptions.Timeout):
-        b.wait_on_status("Queued")
-
-
-def test_global_var(global_var):
-    assert pytest.start_time == 0
-
-
 @pytest.fixture
 def build(build_attrs):
     return tuxbuild.build.Build(**build_attrs)
@@ -236,6 +231,35 @@ class TestBuild:
     def test_client_token(self, build):
         assert type(build.client_token) is str
 
+    def test_submit_build_environment(self, build, build_attrs, mocker):
+        build_attrs["environment"] = {
+            "KCONFIG_ALLCONFIG": "arch/arm64/configs/defconfig",
+        }
+        post_request = mocker.patch("tuxbuild.build.post_request")
+        api_build_url = build_attrs["kbapi_url"] + "/build"
+
+        build.git_ref = None
+        build.git_sha = "badbee"
+        build.environment = {
+            "KCONFIG_ALLCONFIG": "arch/arm64/configs/defconfig",
+        }
+        build.build()
+        post_request.assert_called_with(
+            api_build_url,
+            mocker.ANY,
+            [
+                {
+                    "git_repo": build_attrs["git_repo"],
+                    "git_sha": "badbee",
+                    "toolchain": build_attrs["toolchain"],
+                    "target_arch": build_attrs["target_arch"],
+                    "kconfig": [build_attrs["kconfig"]],
+                    "client_token": mocker.ANY,
+                    "environment": build_attrs["environment"],
+                }
+            ],
+        )
+
 
 class TestWatch:
     @staticmethod
@@ -248,8 +272,36 @@ class TestWatch:
 
 class TestBuildWatch(TestWatch):
     @pytest.fixture(autouse=True)
-    def get_status(self, mocker):
-        return mocker.patch("tuxbuild.build.Build.get_status")
+    def set_build_key(self, build):
+        build.build_key = "0123456789"
+        return build
+
+    @pytest.fixture
+    def build_statuses(self):
+        return [
+            {
+                "tuxbuild_status": "queued",
+                "status_message": "Queued",
+                "git_short_log": "Bla bla bla",
+            },
+            {
+                "tuxbuild_status": "building",
+                "status_message": "Building ...",
+                "git_short_log": "Bla bla bla",
+            },
+            {
+                "tuxbuild_status": "complete",
+                "status_message": "Building ...",
+                "git_short_log": "Bla bla bla",
+                "build_status": "pass",
+                "warnings_count": 0,
+                "errors_count": 0,
+            },
+        ]
+
+    @pytest.fixture(autouse=True)
+    def get_request(self, mocker, build_statuses):
+        return mocker.patch("tuxbuild.build.get_request", side_effect=build_statuses)
 
     def test_watch(self, build):
         watch = iter(build.watch())
@@ -259,78 +311,87 @@ class TestBuildWatch(TestWatch):
         s2 = next(watch)
         assert s2.state == "building"
 
-        build.status["tuxbuild_status"] = "completed"
+        build.status["tuxbuild_status"] = "complete"
         build.status["build_status"] = "pass"
         build.status["warnings_count"] = 0
         s3 = next(watch)
-        assert s3.state == "completed"
+        assert s3.state == "complete"
 
     def test_watch_pass(self, build):
-        build.status["build_status"] = "pass"
-        build.status["warnings_count"] = 0
-
         states = self.watch(build)
         assert len(states) > 1
         state = states[-1]
-        assert state.state == "completed"
+        assert state.state == "complete"
         assert state.status == "pass"
         assert state.warnings == 0
 
-    def test_watch_pass_warnings(self, build):
-        build.status["build_status"] = "pass"
-        build.status["warnings_count"] = 5
+    def test_watch_pass_warnings(self, build, build_statuses):
+        build_statuses[-1]["warnings_count"] = 5
 
         state = self.watch(build)[-1]
         assert "Pass (5 warnings)" in state.message
         assert state.warnings == 5
 
-    def test_watch_pass_one_warning(self, build):
-        build.status["build_status"] = "pass"
-        build.status["warnings_count"] = 1
+    def test_watch_pass_one_warning(self, build, build_statuses):
+        build_statuses[-1]["warnings_count"] = 1
 
         state = self.watch(build)[-1]
         assert "Pass (1 warning)" in state.message
         assert state.warnings == 1
 
-    def test_watch_fail(self, build):
-        build.status["build_status"] = "fail"
-        build.status["errors_count"] = 5
+    def test_watch_fail(self, build, build_statuses):
+        build_statuses[-1]["build_status"] = "fail"
+        build_statuses[-1]["errors_count"] = 5
 
         state = self.watch(build)[-1]
         assert "Fail (5 errors)" in state.message
         assert state.errors == 5
 
-    def test_watch_fail_1_error(self, build):
-        build.status["build_status"] = "fail"
-        build.status["errors_count"] = 1
+    def test_watch_fail_1_error(self, build, build_statuses):
+        build_statuses[-1]["build_status"] = "fail"
+        build_statuses[-1]["errors_count"] = 1
 
         state = self.watch(build)[-1]
         assert "Fail (1 error)" in state.message
         assert state.errors == 1
 
-    def test_watch_fail_status_message(self, build):
-        build.status["build_status"] = "fail"
-        build.status["errors_count"] = 1
-        build.status["status_message"] = "failed to foo the bar"
+    def test_watch_fail_status_message(self, build, build_statuses):
+        build_statuses[-1]["build_status"] = "fail"
+        build_statuses[-1]["errors_count"] = 1
+        build_statuses[-1]["status_message"] = "failed to foo the bar"
 
         state = self.watch(build)[-1]
         assert "with status message 'failed to foo the bar'" in state.message
 
-    def test_watch_not_completed(self, build, mocker):
-        build.status["build_status"] = None
-        build.status["tuxbuild_status"] = "error"
-        build.status["status_message"] = "the infrastructure failed"
+    def test_watch_not_completed(self, build, mocker, build_statuses):
+        build_statuses[-1]["build_status"] = None
+        build_statuses[-1]["tuxbuild_status"] = "error"
+        build_statuses[-1]["status_message"] = "the infrastructure failed"
+        build_statuses.append(build_statuses[0])
+        build_statuses.append(build_statuses[1])
+        build_statuses.append(build_statuses[2])
+        build_statuses.append(build_statuses[0])
+        build_statuses.append(build_statuses[1])
+        build_statuses.append(build_statuses[2])
+
         mocker.patch("tuxbuild.build.Build.build")
         state = self.watch(build)[-1]
-        assert state.state != "completed"
+        assert state.state != "complete"
         assert state.status is None
         assert "the infrastructure failed" in state.message
 
-    def test_retries_on_errors(self, build, mocker):
+    def test_retries_on_errors(self, build, mocker, build_statuses):
+        build_statuses[-1]["build_status"] = None
+        build_statuses[-1]["tuxbuild_status"] = "error"
+        build_statuses[-1]["status_message"] = "the infrastructure failed"
+        build_statuses.append(build_statuses[0])
+        build_statuses.append(build_statuses[1])
+        build_statuses.append(build_statuses[2])
+        build_statuses.append(build_statuses[0])
+        build_statuses.append(build_statuses[1])
+        build_statuses.append(build_statuses[2])
         build_build = mocker.patch("tuxbuild.build.Build.build")
-        build.status["build_status"] = None
-        build.status["tuxbuild_status"] = "error"
-        build.status["status_message"] = "the infrastructure failed"
+
         states = self.watch(build)
         assert build_build.call_count == 2
         assert [state.state for state in states] == [
@@ -345,41 +406,15 @@ class TestBuildWatch(TestWatch):
             "error",
         ]
 
-    def test_retry_succeeds_in_the_second_attempt(self, build, mocker, get_status):
-        statuses = iter(
-            [
-                {
-                    "tuxbuild_status": "queued",
-                },
-                {
-                    "tuxbuild_status": "building",
-                },
-                {
-                    "tuxbuild_status": "error",
-                },
-                {
-                    "tuxbuild_status": "queued",
-                },
-                {
-                    "tuxbuild_status": "building",
-                },
-                {
-                    "tuxbuild_status": "complete",
-                    "build_status": "pass",
-                    "warnings_count": 0,
-                    "errors_count": 0,
-                },
-            ]
-        )
-
-        def __get_status__():
-            nonlocal build
-            nonlocal statuses
-            build.status = next(statuses)
-
-        get_status.side_effect = __get_status__
+    def test_retry_succeeds_in_the_second_attempt(self, build, mocker, build_statuses):
+        complete = build_statuses[-1].copy()
+        build_statuses[-1]["build_status"] = None
+        build_statuses[-1]["tuxbuild_status"] = "error"
+        build_statuses[-1]["status_message"] = "the infrastructure failed"
+        build_statuses.append(build_statuses[0])
+        build_statuses.append(build_statuses[1])
+        build_statuses.append(complete)
         build_build = mocker.patch("tuxbuild.build.Build.build")
-        mocker.patch("time.sleep")
 
         states = self.watch(build)
         assert build_build.call_count == 1
@@ -392,6 +427,33 @@ class TestBuildWatch(TestWatch):
             "complete",
         ]
 
+    def test_from_queued_directly_to_completed(self, build, mocker, build_statuses):
+        build_statuses.pop(1)
+        states = self.watch(build)
+        assert [s.state for s in states] == ["queued", "complete"]
+
+    def test_timeout(self, build, mocker, get_request):
+        get_request.side_effect = iter(lambda: {"tuxbuild_status": "queued"}, 1)
+        with pytest.raises(tuxbuild.exceptions.Timeout):
+            self.watch(build)
+
+    def test_resists_unknown_state(self, build, build_statuses):
+        build_statuses.insert(
+            2,
+            {
+                "tuxbuild_status": "spiralling",
+                "status_message": "Spiralling out of control",
+                "git_short_log": "Bla bla bla",
+            },
+        )
+        states = self.watch(build)
+        assert [s.state for s in states] == [
+            "queued",
+            "building",
+            "spiralling",
+            "complete",
+        ]
+
     def test_output_with_multiple_kconfigs(self, build):
         build.kconfig = ["defconfig", "https://raw.foo.com/kconfig/myconfig.txt"]
         assert "(defconfig+1)" in str(build)
