@@ -132,6 +132,7 @@ def test_build_git_head(mocker, tuxbuild_config):
         kconfig=("defconfig",),
         kconfig_allconfig=None,
         toolchain="gcc-9",
+        environment=(),
     )
     wait_for_object.assert_called()
     assert result.exit_code == 0
@@ -450,3 +451,52 @@ def test_wait_for_object_infra_failure_retried(mocker):
     build = mocker.MagicMock()
     build.watch.return_value = [error, retry_pass]
     assert tuxbuild.cli.wait_for_object(build)
+
+
+def test_build_environment_option():
+    """ Test calling build with --help to see environment option"""
+    runner = CliRunner()
+    result = runner.invoke(tuxbuild.cli.build, ["--help"])
+    assert result.exit_code == 0
+    assert "-e, --environment KEY_VALUE" in result.output
+
+
+def test_build_valid_environment(mocker, tuxbuild_config):
+    Build = mocker.patch("tuxbuild.Build")
+    build = Build.return_value
+    wait_for_object = mocker.patch("tuxbuild.cli.wait_for_object")
+    runner = CliRunner()
+    result = runner.invoke(
+        tuxbuild.cli.build,
+        [
+            "--git-repo=https://git.example.com/linux.git",
+            "--git-ref=master",
+            "--target-arch=arm64",
+            "--kconfig=defconfig",
+            "--toolchain=gcc-9",
+            "-e KCONFIG_ALLCONFIG=arch/arm64/configs/defconfig",
+        ],
+    )
+    assert result.exit_code == 0
+    assert build.build.call_count == 1
+    assert wait_for_object.call_count == 1
+
+
+def test_invalid_environment_key_value(mocker, tuxbuild_config):
+    Build = mocker.patch("tuxbuild.Build")
+    build = Build.return_value
+    runner = CliRunner()
+    result = runner.invoke(
+        tuxbuild.cli.build,
+        [
+            "--git-repo=https://git.example.com/linux.git",
+            "--git-ref=master",
+            "--target-arch=arm64",
+            "--kconfig=defconfig",
+            "--toolchain=gcc-9",
+            "-e INVALID",
+        ],
+    )
+    assert result.exit_code == 1
+    assert build.build.call_count == 0
+    assert "Key Value pair not valid:  INVALID" in result.output
