@@ -4,6 +4,7 @@ from attr import attrs, attrib
 import concurrent.futures
 import json
 import queue
+import os
 import random
 import re
 import time
@@ -13,7 +14,7 @@ import uuid
 
 # We will poll for status change for an average duration of 180 minutes
 state_timeout = 10800  # 60 * 180
-delay_status_update = 30
+delay_status_update = int(os.getenv("TUXBUILD_DELAY_STATUS_UPDATE", "29")) + 1
 
 
 def post_request(url, headers, request):
@@ -62,6 +63,7 @@ class Build:
     token = attrib()
     kbapi_url = attrib()
     kconfig_allconfig = attrib(default=None)
+    environment = attrib(default={})
 
     build_data = attrib(default=None)
     build_key = attrib(default=None)
@@ -115,6 +117,8 @@ class Build:
         build_entry["target_arch"] = self.target_arch
         build_entry["toolchain"] = self.toolchain
         build_entry["kconfig"] = self.kconfig
+        if self.environment:
+            build_entry["environment"] = dict(self.environment)
         if self.kconfig_allconfig is not None:
             build_entry["kconfig_allconfig"] = self.kconfig_allconfig
         return build_entry
@@ -147,65 +151,77 @@ class Build:
         url = self.kbapi_url + "/status/" + self.build_key
         self.status = get_request(url, self.headers)
 
-    def wait_on_status(self, status):
-        """
-        Wait until the given status changes
-
-        For example, if status is 'queued', wait_for_status
-        will return once the status is no longer 'queued'
-
-        Will timeout after state_timeout
-        """
-        timeout = time.time() + state_timeout
-
-        while time.time() < timeout:
-            self.get_status()
-            if self.status["tuxbuild_status"] != status:
-                break
-            time.sleep(random.randrange(delay_status_update))
-        else:
-            raise tuxbuild.exceptions.Timeout(
-                f"Build timed out after {state_timeout/60} minutes; abandoning"
-            )
-
     def watch(self):
+        """
+        Wait for the build to finish. Whenever the state changes, yielding the
+        new state).
+
+        Will timeout after `state_timeout` seconds.
+        """
         attempts = 1
         max_attempts = 3
+        waiting_states = {
+            "queued": BuildState(
+                build=self,
+                state="queued",
+                message="Queued",
+                icon="⏳",
+                cli_color="white",
+            ),
+            "provisioning": BuildState(
+                build=self,
+                state="provisioning",
+                message="Provisioning",
+                icon="⚙️",
+                cli_color="white",
+            ),
+            "building": BuildState(
+                build=self,
+                state="building",
+                message="Building",
+                icon="🚀",
+                cli_color="cyan",
+            ),
+        }
+        final_states = ["complete", "error"]
+        timeout = time.time() + state_timeout
+        previous_state = None
         while True:
-            waiting_states = [
-                BuildState(
-                    build=self,
-                    state="queued",
-                    message="Queued",
-                    icon="⏳",
-                    cli_color="white",
-                ),
-                BuildState(
-                    build=self,
-                    state="building",
-                    message="Building",
-                    icon="⚗️ ",
-                    cli_color="cyan",
-                ),
-            ]
-            for state in waiting_states:
-                yield state
-                self.wait_on_status(state.state)
-
-            if self.status["tuxbuild_status"] == "error" and attempts < max_attempts:
-                attempts += 1
-                message = self.status.get("status_message", "infrastructure error")
-                yield BuildState(
-                    build=self,
-                    state=self.status["tuxbuild_status"],
-                    message=message
-                    + f" - retrying (attempt {attempts}/{max_attempts})",
-                    icon="🔧",
-                    cli_color="yellow",
+            self.get_status()
+            state = self.status["tuxbuild_status"]
+            if state != previous_state:
+                if state in waiting_states:
+                    yield waiting_states[state]
+                elif state == "error" and attempts < max_attempts:
+                    attempts += 1
+                    message = self.status.get("status_message", "infrastructure error")
+                    yield BuildState(
+                        build=self,
+                        state=self.status["tuxbuild_status"],
+                        message=message
+                        + f" - retrying (attempt {attempts}/{max_attempts})",
+                        icon="🔧",
+                        cli_color="yellow",
+                    )
+                    self.build()
+                elif state in final_states:
+                    break
+                else:
+                    # unknown state
+                    yield BuildState(
+                        build=self,
+                        state=state,
+                        message=self.status.get("status_message", state),
+                        icon="🎰",
+                        cli_color="yellow",
+                    )
+            elif time.time() >= timeout:
+                raise tuxbuild.exceptions.Timeout(
+                    f"Build timed out after {state_timeout/60} minutes; abandoning"
                 )
-                self.build()
-            else:
-                break
+            delay = random.randrange(delay_status_update)
+            time.sleep(delay)
+            previous_state = state
 
         errors = 0
         warnings = 0
