@@ -38,7 +38,7 @@ def wait_for_object(build_object):
         msg = click.style(
             f"{state.icon} {state.message}: ", fg=state.cli_color, bold=True
         ) + str(state.build)
-        if state.errors or state.state == "error":
+        if state.status == "fail" or state.state == "error":
             warning(msg)
             if state.final:
                 result = False
@@ -49,6 +49,13 @@ def wait_for_object(build_object):
     return result
 
 
+def key_value(s):
+    if "=" not in s:
+        error(f"Key Value pair not valid: {s}")
+    parts = s.split("=")
+    return (parts[0], "=".join(parts[1:]))
+
+
 @click.group(name="tuxbuild")
 @click.version_option()  # Implement --version
 def cli():
@@ -76,14 +83,17 @@ def common_options(required):
         ),
         option(
             "--target-arch",
-            help="Target architecture [arc|arm|arm64|hppa|i386|mips|powerpc|riscv|s390|sh|sparc|x86_64]",
+            help="Target architecture [arc|arm|arm64|i386|mips|parisc|powerpc|riscv|s390|sh|sparc|x86_64]",
         ),
         option(
             "--kconfig",
             multiple=True,
             help="Kernel kconfig arguments (may be specified multiple times)",
         ),
-        option("--toolchain", help="Toolchain [gcc-8|gcc-9|gcc-10|clang-10|clang-11]"),
+        option(
+            "--toolchain",
+            help="Toolchain [gcc-8|gcc-9|gcc-10|clang-10|clang-11|clang-nightly]",
+        ),
         option(
             "--kconfig-allconfig",
             help=(
@@ -123,6 +133,13 @@ def common_options(required):
             is_flag=True,
             help="Prints build logs to stderr in case of warnings or errors",
         ),
+        option(
+            "-e",
+            "--environment",
+            type=key_value,
+            multiple=True,
+            help="Set environment variables for the build. Format: KEY=VALUE",
+        ),
     ]
 
     def wrapper(f):
