@@ -8,6 +8,16 @@ class TuxbuildError(Exception):
     error_type = ""
 
 
+class CantGetConfiguration(TuxbuildError):
+    error_help = "Problem reading configuration"
+    error_type = "configuration"
+
+
+class InvalidConfiguration(TuxbuildError):
+    error_help = "Invalid configuration"
+    error_type = "configuration"
+
+
 class TokenNotFound(TuxbuildError):
     error_help = "No token provided"
     error_type = "Configuration"
@@ -23,6 +33,11 @@ class BadRequest(TuxbuildError):
     error_type = "API"
 
 
+class InvalidToken(TuxbuildError):
+    error_help = "The provided token was not accepted by the server"
+    error_type = "API"
+
+
 class Timeout(TuxbuildError):
     error_help = "A tuxbuild API call failed"
     error_type = "API"
